// Copyright 2021 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package a

type A[T any] struct {
	a int
}

func (a A[T]) F() {
	_ = &a.a
}
