// Copyright 2021 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package b

import "./a"

func F2() int {
	var mia a.MyIntAlias
	return mia.Get()
}
