// Copyright 2015 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

//go:build gccgo
// +build gccgo

package main

func useVFPv1() {}

func useVFPv3() {}

func useARMv6K() {}
