// rundir

// Copyright 2012 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

// Issue 4159: exported inlinable functions squash
// complex literals "a+bi" to "a+b".

package ignored

