// Copyright 2022 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package main

import (
	"./a"
	"fmt"
)

func BuildInt() int {
	return a.BuildInt()
}

func main() {
	if got, want := BuildInt(), 0; got != want {
		panic(fmt.Sprintf("got %d, want %d", got, want))
	}
}
