// compile

// Copyright 2010 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

// Test that a comment ending a source file does not need a final newline.
// Compiles but does not run.

package eof1

// No newline at the end of this comment.